import pygame
import math
import random
from data import loadfont


class Text:

    def __init__(self, text, font="default", size=12, pos=(0, 0), hilite=False):
        if font == "default":
            font = "OpenSans-CondBold.ttf"
        if font is not None:
            self.font = loadfont(font, size)
        else:
            self.font = pygame.font.Font(None, size)
        self.text = text
        self.fg = [255, 255, 255, 255]
        self.bg = [0, 0, 0, 0]
        self.colors = [
            [231, 76, 60, 255],
            [236, 240, 241, 255],
            [41, 128, 185, 255]
        ]
        self.colorset = 0
        self.regen()
        self.offsets = []
        self.angles = []
        self.pos = pos
        self.waveset = random.random() * 10
        self.wavelength = 3
        self.amplitude = 2
        self.wavespeed = .7
        self.hilite = False
        self.boring = False
        self.w = 0
        self.total = float(len(self.letter_surfs))
        for i, s in enumerate(self.letter_surfs):
            proportion = float(i) / self.total
            self.offsets.append([
                self.w,
                self.wave(proportion)
            ])
            self.w += s.get_width()
        self.recompute_size()
        self.hilite = hilite

    def recompute_size(self):
        size = self.font.size(self.text)
        self.line_width = size[0]
        self.line_height = size[1]

    def clear(self):
        self.text = ""
        self.w = 0
        self.offsets = []
        self.regen()

    def regen(self):
        self.letter_surfs = [self.font.render(
            x, True, self.fg
        ) for x in self.text]
        self.recompute_size()

    def append(self, char):
        self.text = "{}{}".format(self.text, char)
        self.letter_surfs.append(self.font.render(char, True, self.fg))
        self.offsets.append([self.w, 0])
        self.w += self.letter_surfs[-1].get_width()
        self.recompute_size()

    def shorten_by_1(self):
        self.text = self.text[:-1]
        old = self.letter_surfs.pop(-1)
        self.offsets.pop(-1)
        self.w -= old.get_width()
        self.recompute_size()

    def update(self, dt):
        if self.boring:
            self.colorset += dt * .4
            if self.colorset > len(self.colors):
                self.colorset = 0
            return

        self.waveset += self.wavespeed * dt
        if self.waveset > 2 * math.pi:
            self.waveset = 0

        for i, _ in enumerate(self.letter_surfs):
            proportion = float(i) / self.total
            self.offsets[i][1] = self.wave(proportion)

        if self.hilite:
            self.colorset += dt * .4
            if self.colorset > len(self.colors):
                self.colorset = 0

    def draw(self, surf):
        for i, l in enumerate(self.letter_surfs):
            x = self.pos[0] + self.offsets[i][0]
            y = self.pos[1] + self.offsets[i][1]
            angle = -self.offsets[i][1]

            s = l.copy()
            if not self.boring:
                s = pygame.transform.rotate(l.copy(), angle)

            if self.hilite:
                csurf = s.copy()
                csurf.fill(
                    self.colors[
                            i % len(self.colors) - math.floor(self.colorset)
                    ]
                )
                s.blit(csurf, (0, 0), None, pygame.BLEND_RGBA_MULT)
                surf.blit(s, (x, y))
            else:
                surf.blit(s, (x, y))

    def wave(self, x):
        return math.sin(
            (math.pi * -self.wavelength * x) + self.waveset
        ) * self.amplitude


class Button:

    def __init__(self, text, rect, action, param=None, size=34, font="default"):
        self.text = Text(text, font=font, size=size, pos=(0, 0))
        xoffset = rect.x + ((rect.width - self.text.line_width) / 2.0)
        yoffset = rect.y + ((rect.height - self.text.line_height) / 2.0)
        self.text.pos = (xoffset, yoffset)
        self.btn_rect = rect
        self.action = action
        self.inactive_color = [0, 0, 0, 255]
        self.hover_color = [35, 35, 35, 255]
        self.color = self.inactive_color
        self.param = param

    def ishover(self):

        return self.btn_rect.collidepoint(pygame.mouse.get_pos())

    def handleclick(self, pos):
        if self.btn_rect.collidepoint(pos):
            if self.param is not None:
                self.action(self.param)
            else:
                self.action()

    def update(self, dt):
        self.text.update(dt)
        if self.ishover():
            self.text.hilite = True
            self.color = self.hover_color
            self.text.wavelength = 2.4
            self.text.amplitude = 4.5
        else:
            self.text.hilite = False
            self.color = self.inactive_color
            self.text.wavelength = 1
            self.text.amplitude = 2

    def draw(self, surf):
        surf.fill(self.color, self.btn_rect)
        self.text.draw(surf)
