import pygame
from fysom import Fysom
from constants import RESOLUTION_X, RESOLUTION_Y, DISPLAY_OPTS
from menu import MainMenu, Options, Credits, Help, StageSelect, CharSelect
from menu import ScoreScreen, Pause
from game import PlayStage
from util import normalize
from data import loadmusic


class Game:

    def __init__(self):
        pygame.init()
        self.screen = pygame.display.set_mode(
            (RESOLUTION_X, RESOLUTION_Y),
            DISPLAY_OPTS
        )
        pygame.display.set_caption('Ms. Information')
        self.fps = 60
        self.time = 0
        self.clock = pygame.time.Clock()
        self.main_loop = True
        self.next_game_level = 0
        self.player_char = 0
        self.last_score = 0
        self.last_streak = 0
        self.state = Fysom({
            "initial": "start",
            "events": [
                {"name": "init_menu",
                    "src": "start", "dst": "main_menu"},
                {"name": "select_stage",
                    "src": [
                        "main_menu",
                        "score_screen",
                        "char_select",
                        "paused"],
                    "dst": "stage_select"},
                {"name": "select_char",
                    "src": [
                        "main_menu",
                        "stage_select"],
                    "dst": "char_select"},
                {"name": "start_game",
                    "src": [
                        "stage_select",
                        "char_select",
                        "score_screen"],
                    "dst": "in_game"},
                {"name": "pause",
                    "src": "in_game", "dst": "paused"},
                {"name": "unpause",
                    "src": "paused", "dst": "in_game"},
                {"name": "win_game",
                    "src": "in_game", "dst": "score_screen"},
                {"name": "lose_game",
                    "src": "in_game", "dst": "score_screen"},
                {"name": "go_title",
                    "src": [
                        "stage_select",
                        "char_select",
                        "paused",
                        "score_screen",
                        "options",
                        "credits",
                        "help"
                    ], "dst": "main_menu"},
                {"name": "go_options",
                    "src": "main_menu", "dst": "options"},
                {"name": "go_credits",
                    "src": "main_menu", "dst": "credits"},
                {"name": "go_help",
                    "src": "main_menu", "dst": "help"}
            ],
            "callbacks": {
                'onchangestate': self.change_state
            }
        })
        self.stage = None
        self.changing = False
        self.stage_ready = False
        self.fade = pygame.Surface((RESOLUTION_X, RESOLUTION_Y)).convert_alpha()
        self.fade_step = 0
        self.unpause = False

    def run(self):
        # self.state.init_menu()
        while self.main_loop:
            dt = self.clock.tick(self.fps) / 100.0
            self.time += dt
            self.update(dt)
            self.draw(self.screen)

    def update(self, dt):

        if self.changing:
            if not self.stage_ready:
                self.fade_step += .1 * dt
                if self.fade_step > 1:
                    self.load_stage()
                    self.stage_ready = True
            else:
                self.fade_step -= .1 * dt
                if self.fade_step < 0:
                    self.changing = False
                    self.stage_ready = False
                    self.fade_step = 0
                self.stage.update(dt)

            self.fade_step = normalize(self.fade_step)
        else:
            if self.stage:
                self.stage.update(dt)

        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                self.main_loop = False

    def draw(self, screen):
        self.screen.fill([0, 0, 0, 255])
        if self.stage:
            self.stage.draw(screen)
        if self.changing:
            self.draw_fade()

        pygame.display.flip()

    def change_state(self, e):
        print("{} -> {}".format(e.src, e.dst))
        self.changing = True
        if e.dst == "paused":
            self.unpause = self.stage
        if e.event == "win_game":
            self.won = True
        if e.event == "lose_game":
            self.won = False

    def draw_fade(self):
        self.fade.fill([0, 0, 0, self.fade_step * 255])
        self.screen.blit(self.fade, (0, 0))

    def load_stage(self):
        s = self.state.current
        if s == "main_menu":
            loadmusic("msinformation_theme.ogg")
            pygame.mixer.music.play(-1)
            self.stage = MainMenu(self)
        if s == "stage_select":
            self.stage = StageSelect(self)
        if s == "char_select":
            self.stage = CharSelect(self)
        if s == "in_game":
            if self.unpause:
                self.stage = self.unpause
                self.unpause = False
            else:
                self.stage = PlayStage(self)
        if s == "options":
            self.stage = Options(self)
        if s == "credits":
            self.stage = Credits(self)
        if s == "help":
            self.stage = Help(self)
        if s == "score_screen":
            self.stage = ScoreScreen(self)
        if s == "paused":
            self.stage = Pause(self)

    def exit(self):
        self.main_loop = False


def test():
    print("test called")

if __name__ == "__main__":
    g = Game()
    g.state.init_menu()
    # g.state.select_char()
    # g.state.select_stage()
    # g.state.start_game()
    g.run()
